@@ -1,4 +1,5 @@
 import os
+import sys
 
 from flask import request, render_template, current_app, url_for
 from flask.views import View
@@ -6,7 +7,8 @@ from flask.views import View
 from werkzeug.wrappers import Response
 from werkzeug.exceptions import NotFound
 
-from dulwich.objects import Blob
+import dulwich.objects
+import dulwich.archive
 
 try:
     import ctags
@@ -16,7 +18,7 @@ else:
     from klaus import ctagscache
     CTAGS_CACHE = ctagscache.CTagsCache()
 
-from klaus import markup, tarutils
+from klaus import markup
 from klaus.highlighting import pygmentize
 from klaus.utils import parent_directory, subpaths, force_unicode, guess_is_binary, \
                         guess_is_image, replace_dupes
@@ -39,6 +41,24 @@ def robots_txt():
     return current_app.send_static_file('robots.txt')
 
 
+def _get_repo_and_rev(repo, rev=None):
+    try:
+        repo = current_app.repos[repo]
+    except KeyError:
+        raise NotFound("No such repository %r" % repo)
+
+    if rev is None:
+        rev = repo.get_default_branch()
+        if rev is None:
+            raise NotFound("Empty repository")
+    try:
+        commit = repo.get_commit(rev)
+    except KeyError:
+        raise NotFound("No such commit %r" % rev)
+
+    return repo, rev, commit
+
+
 class BaseRepoView(View):
     """Base for all views with a repo context.
 
@@ -63,20 +83,7 @@ class BaseRepoView(View):
         return render_template(self.template_name, **self.context)
 
     def make_template_context(self, repo, rev, path):
-        try:
-            repo = current_app.repos[repo]
-        except KeyError:
-            raise NotFound("No such repository %r" % repo)
-
-        if rev is None:
-            rev = repo.get_default_branch()
-            if rev is None:
-                raise NotFound("Empty repository")
-        try:
-            commit = repo.get_commit(rev)
-        except KeyError:
-            raise NotFound("No such commit %r" % rev)
-
+        repo, rev, commit = _get_repo_and_rev(repo, rev)
         try:
             blob_or_tree = repo.get_blob_or_tree(commit, path)
         except KeyError:
@@ -123,7 +130,7 @@ class TreeViewMixin(object):
 
     def get_root_directory(self):
         root_directory = self.context['path']
-        if isinstance(self.context['blob_or_tree'], Blob):
+        if isinstance(self.context['blob_or_tree'], dulwich.objects.Blob):
             # 'path' is a file (not folder) name
             root_directory = parent_directory(root_directory)
         return root_directory
@@ -177,7 +184,7 @@ class HistoryView(TreeViewMixin, BaseRepoView):
 class BaseBlobView(BaseRepoView):
     def make_template_context(self, *args):
         super(BaseBlobView, self).make_template_context(*args)
-        if not isinstance(self.context['blob_or_tree'], Blob):
+        if not isinstance(self.context['blob_or_tree'], dulwich.objects.Blob):
             raise NotFound("Not a blob")
         self.context['filename'] = os.path.basename(self.context['path'])
 
@@ -201,7 +208,7 @@ class BaseFileView(TreeViewMixin, BaseBlobView):
                 self.context['commit'].id
             )
             ctags_args = {
-                'ctags': ctags.CTags(ctags_tagsfile),
+                'ctags': ctags.CTags(ctags_tagsfile.encode(sys.getfilesystemencoding())),
                 'ctags_baseurl': ctags_base_url,
             }
         else:
@@ -287,7 +294,7 @@ class DownloadView(BaseRepoView):
             'Cache-Control': "no-store",  # Disables browser caching
         }
 
-        tar_stream = tarutils.tar_stream(
+        tar_stream = dulwich.archive.tar_stream(
             self.context['repo'],
             self.context['blob_or_tree'],
             self.context['commit'].commit_time,
